using System;
using System.Threading;

namespace ThreadEventTest
{
	/// <summary>
	/// A worker thread that counts and emits an event with a status string in EventArgs
	/// </summary>

	public class Worker
	{
		private bool running;
		private int counter;

		// Because we want to update the UI with actual data--a string--we need to create
		// our own EventArgs-derived class.  Form1 owns the event handler and will use
		// this as Worker.WorkerEventArgs, whereas here it is simply WorkerEventArgs.
		public class WorkerEventArgs : EventArgs 
		{
			public string status;
			public WorkerEventArgs(string s) 
			{
				status = s;
			}
		}

		// Declare a delegate class to hold the prototype information we want to enforce
		// for methods which subscribe to the event we'll define.  Because this event is
		// an ordinary WinForms type event, it has to take a source object and an
		// EventArgs-derived structure.
		public delegate void WorkerOutputHandler(object worker, WorkerEventArgs e);
		// And here we use the delegate--the prototype information--to declare an actual
		// event object.  OnWorkerOutput is a field in Worker, which subscribers can
		// compose (+=) their event handlers onto.
		public event WorkerOutputHandler OnWorkerOutput;

		public Worker()
		{
			running = false;
			counter = 0;
		}

		// Here is the lengthy processing.  Every 100 counts, we raise the OnWorkerOutput
		// event, until 'running' becomes false.  That will happen when the user clicks
		// the UI thread's StopButton, whose handler calls Worker.Stop().
		public void Go() 
		{
			running = true;
			while(running) 
			{
				counter++;
				if(counter%100 == 0) // time to raise the event.
				{
					// First, create the WorkerEventArgs object to hold the string we
					// want to pass back to the UI.
					WorkerEventArgs e = new WorkerEventArgs("count on thread " + 
															Thread.CurrentThread.Name +
															" = " + counter + "\n");
					// now, if anyone has subscribed to the event,
					if(OnWorkerOutput != null)
						// then raise the it.  Event handlers are responsible for putting
						// themselves into the right thread.
						OnWorkerOutput(this,e);	
				}
				Thread.Sleep(10);
			}
		}

		/// <summary>
		/// Stop the worker thread by setting the 'running' flag to false; the
		/// processing loop will notice this and exit.
		/// </summary>
		public void Stop()
		{
			running = false;
		}
	}
}
